#!/bin/bash
# Virtual Central Lock installer
# Copyright (C) Tosibox Oy 2020

VERSION="0.1.6"

EULA="https://www.tosibox.com/uploads/2018/05/Tosibox-EULA.pdf"

OPTIONS="TOSIBOX(R) Virtual Central Lock - Command line installer $VERSION
usage: $0 [COMMAND][OPTIONS]
Commands:
install              Deploy TOSIBOX(R) Virtual Central Lock
uninstall            Delete TOSIBOX(R) Virtual Central Lock
activate             Activate TOSIBOX(R) Virtual Central Lock
match                Show remote matching code

Deployment options: install/uninstall
-n, --name [name]                   Name of the deployment
-r, --variant [variant]             Variant [aws|az]
-i, --source-ip [CIDR/auto]         CIDR or 'auto' for automatic from ipinfo.io to
                                    restricting activation traffic only to your IP.
                                    NOTICE: activation traffic is unencrypted and anyone
                                    in your local network or in the network path could
                                    eavesdrop the traffic.
Activation options:
-l, --license [activation code]     Activation code from Tosibox

Generic options:
-h, --help                          Show this help
-v, --version                       Show version
--dryrun                            Dry run (only echo the commands to be run)
"

RED='\033[0;31m'
NC='\033[0m' # No Color
DRYRUN_ECHO="echo -e ${RED} "
DR=
AZURE_DIR="az"
AWS_DIR="aws"
GCP_DIR="gcp"

AWS_INSTANCE_ID=

LOCATION=
AWS_AVAILABILITY_ZONE=

log() {
  echo -e "\033[0m$@"
}

log-ne() {
  echo -ne "\033[0m$@"
}

if [ -z "$1" ]; then
  echo "$OPTIONS"
  exit 1
fi

NAME=""
LICENSE=""
VARIANT=""

ACTIVATE=
MATCH=
INSTALL=
UNINSTALL=

_AWS_PROFILE=

while [[ "$#" -gt 0 ]]; do case $1 in
  install) INSTALL=1; ACTIVATE=1;;
  -r|--variant) VARIANT="$2"; shift;;
  -n|--name) NAME="$2"; shift;;
  -i|--source-ip) SOURCE_IP="$2"; shift;;
  uninstall) UNINSTALL=1;;
  activate) ACTIVATE=1;;
  -l|--license) LICENSE="$2"; shift;;
  match) MATCH=1;;

  -h|--help) echo "$OPTIONS";exit 0; shift;;
  -v|--version) echo "$VERSION";exit 0; shift;;
  --dryrun) DR=$DRYRUN_ECHO;;

  *) echo "Unknown parameter passed: $1"; exit 1;;
esac; shift;
done


MD5="md5"
[ -x "$(command -v md5sum)" ] && MD5=md5sum   # MacOS

log "Checking dependencies... "
deps=0
for name in strings awk sed grep $MD5 tail head curl cut sleep cp tr rm
do
  [ -x "$(command -v $name)" ] || { log "$name command missing.";deps=1; }
done
[ $deps -ne 1 ] && log "OK" || { log "Please install the above commands and rerun this script";exit 1; }


# function for checking IP adderss validity
valid_ip() {
  local  ip=$1
  local  stat=1

  if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
    OIFS=$IFS
    IFS='.'
    ip=($ip)
    IFS=$OIFS
    [[ ${ip[0]} -le 255 && ${ip[1]} -le 255 \
       && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]
      stat=$?
  fi
  return $stat
}

get_aws_profile() {
  if [ $AWS_PROFILE ]; then
    log "Your AWS profile is:" $AWS_PROFILE
    _AWS_PROFILE=$AWS_PROFILE
  else
    log  "You need to create AWS profile in AWS console"
    read -p "Enter your AWS Profile: " aws_prof
    _AWS_PROFILE=$aws_prof
  fi
}


install_aws() {
  [ -x "$(command -v aws)" ] || { log "Please install AWS client and try again:"; log " - Linux: apt-get install awscli"; log " - MacOS: brew install awscli"; exit 1; }

  log "Creating an AWS CloudFormation deployment for TOSIBOX(R) Virtual Central Lock"

  cd $AWS_DIR

  get_aws_profile
  get_location


  deployment=$NAME #"-stack"
  bucket=$NAME #"-bucket"
  prefix=$NAME #"-prefix"

  log-ne "Finding AMI id for region: "
  AMI_ID=$(aws --profile $_AWS_PROFILE --region $LOCATION ec2 describe-images --owners 366737907935 --filters "Name=name,Values=TBVCL*" | grep ImageId | awk -F: '{gsub(/ *"*,*/, "", $2); print $2}' | sort -r | head -n 1)
  [ -z $AMI_ID ] && log "No suitable AMI for this region. Contact support@tosibox.com for help or choose another region. Ensure you have the latest install script version." && exit 1
  log $AMI_ID

  log "Creating deployment bucket.."
  $DR aws --profile $_AWS_PROFILE --region $LOCATION s3 mb "s3://$bucket" \
  || { log "Creating bucket failed. Exiting." && exit 1; }


  log "Packaging deployment.."
  $DR aws --profile $_AWS_PROFILE --region $LOCATION cloudformation package \
  --template-file "$yamlfile" \
  --output-template packaged-template.yaml \
  --s3-bucket "$bucket" \
  || { log "Packaging failed. Exiting." >&2 && exit 1; }

  log "Creating deployment.."
  $DR aws --profile $_AWS_PROFILE --region $LOCATION cloudformation deploy \
  --template-file packaged-template.yaml \
  --stack-name "$deployment" \
  --parameter-overrides Prefix="$prefix" SourceIPAddressRange="$SOURCE_IP" AvailabilityZone="$AWS_AVAILABILITY_ZONE" AMI="$AMI_ID" \
  || { log "Deployment failed. Please verify that you have the latest version of the script. Exiting." && exit 1; }

  cd ..
}

azure_blob_copying() {
    local PROGRESS=$((
        $($DR az storage blob show -n $VHD_FILE \
            -c $AZ_STORAGE_CONTAINER \
            --account-name $AZ_STORAGE_ACCOUNT \
            --account-key $AZ_STORAGE_KEY \
            | grep progress \
            | awk '{ print $NF }' \
            | tr -d '"' \
            | tr -d "[:space:]" \
            | tr -d "," \
            | sed "s/\//*100\//"
    )
    ))
    log-ne "Progress: $PROGRESS%\r"
    [ "$PROGRESS" != "100" ]
}


install_azure() {

  [ -x "$(command -v az)" ] || { log "Please install the latest Azure client and try again:"; log " - Linux / WSL2: (https://docs.microsoft.com/en-us/cli/azure/install-azure-cli)"; log " - MacOS: brew update && brew install azure-cli"; exit 1; }

  log "Creating an Azure deployment for TOSIBOX(R) Virtual Central Lock"

  cd $AZURE_DIR

  # Do azure login if necessary, print subscriptions

  while ! az account show --output table ; do
     az login
  done

  get_location

  AZ_DEF_SUB=$(az account show | grep '"id":' | head -1 | awk -F: '{gsub(/ *"*,*/, "", $2); print $2}')
  # Pick subscription to use for deployment
  while ! az account show --subscription $AZ_SUB &>/dev/null ; do
      read -p "Enter subscription ID to use (default: $AZ_DEF_SUB): " AZ_SUB
      [ -z $AZ_SUB ] && AZ_SUB=$AZ_DEF_SUB
  done

  VCL_DL_URL="https://downloads.tosibox.com/VCL/TOSIBOX_Virtual_Central_Lock_latest_cloud.vhd"
  SAS_URI="https://vclstorage.blob.core.windows.net/vhdstorage/TOSIBOX_Virtual_Central_Lock_latest_cloud.vhd"

  curl -Is $SAS_URI >/dev/null \
  || { log "Virtual Central Lock disk image is not available in this region. Download the image from Tosibox website, and manually edit the template. You may contact support@tosibox.com for more instructions." && exit 1; }

  resource_group="$NAME"

  cp vcl-rg-parameters.json tmp_rg_parameters.json
  cp vcl-vm-parameters.json tmp_vcl_parameters.json

  sed -i -e "s/%%resourceGroup%%/$resource_group/g" tmp_rg_parameters.json
  sed -i -e "s/%%resourceGroup%%/$resource_group/g" tmp_vcl_parameters.json

  sed -i -e "s/%%subscriptionID%%/$AZ_SUB/g" tmp_rg_parameters.json
  sed -i -e "s/%%subscriptionID%%/$AZ_SUB/g" tmp_vcl_parameters.json

  sed -i -e "s/%%location%%/$LOCATION/g" tmp_rg_parameters.json
  sed -i -e "s/%%location%%/$LOCATION/g" tmp_vcl_parameters.json

  sed -i -e "s;%%sourceIP%%;$SOURCE_IP;g" tmp_rg_parameters.json
  sed -i -e "s;%%sourceIP%%;$SOURCE_IP;g" tmp_vcl_parameters.json

  log "Creating deployment template..."
  $DR az deployment sub create --name "$NAME" --template-file vcl-rg.json --location $LOCATION --parameters @tmp_rg_parameters.json >/dev/null \
    || { log "Deployment creation failed, please check parameters." && exit 1; }


  ########################################################
  # this part required until we are in Azure marketplace #
  ########################################################

  AZ_STORAGE_ACCOUNT=$(echo $NAME"storage" | cut -c 1-24)
  AZ_STORAGE_CONTAINER="vhdstorage"
  VHD_FILE="TOSIBOX_Virtual_Central_Lock_latest_cloud.vhd"
  AZ_STORAGE_ACCOUNT_URI="https://$AZ_STORAGE_ACCOUNT.blob.core.windows.net"
  AZ_GROUP=$resource_group

  log "Creating storage account for VHD..."
  $DR az storage account create --output table \
        --resource-group $AZ_GROUP \
        --location $LOCATION \
        --name "$AZ_STORAGE_ACCOUNT" \
        --kind Storage \
        --sku Standard_LRS \
    || { log "Storage account creation failed, please check parameters." && exit 1; }

  AZ_STORAGE_KEY=$(
    $DR az storage account keys list --output table \
        --resource-group $AZ_GROUP \
        --account-name $AZ_STORAGE_ACCOUNT |
        grep "key1" |
        awk '{ print $NF }'
    )

  log "Creating blob container for VHD..."
  $DR az storage container create --output table \
        --name "$AZ_STORAGE_CONTAINER" \
        --account-name "$AZ_STORAGE_ACCOUNT" \
        --account-key "$AZ_STORAGE_KEY" \
        2>/dev/null \
    || { log "Storage container creation failed, please check parameters." && exit 1; }

  VHD_PATH="./$VHD_FILE"

  log "Copying blob to the VHD container from URL..."
  $DR az storage blob copy start \
      --account-name $AZ_STORAGE_ACCOUNT \
      --destination-blob $VHD_FILE \
      --destination-container $AZ_STORAGE_CONTAINER \
      --source-uri $SAS_URI \
      --output table \
      --account-key $AZ_STORAGE_KEY \
  || {
  log "Storage blob copy failed, downloading locally from Tosibox URL."
  log
  log "Downloading 8GB file from $VCL_DL_URL.. This may take a long time depending on your connection."
  $DR curl $VCL_DL_URL --output $VHD_PATH

  log "Uploading blob to the VHD container from local file..."
  $DR az storage blob upload --account-name $AZ_STORAGE_ACCOUNT \
      --account-key $AZ_STORAGE_KEY \
      --container-name $AZ_STORAGE_CONTAINER \
      --type page \
      --file "$VHD_PATH" \
      --name "$VHD_FILE" \
      || "Storage blob copy failed, downloading locally from Tosibox URL." && exit 1
  }

  log
  log "Copying blob to storage container. This step may take up to 15-30 minutes."
  while azure_blob_copying; do
       sleep 3 &>/dev/null
  done
  log
  log "Blob copy finished."

  SAS_URI="$AZ_STORAGE_ACCOUNT_URI/$AZ_STORAGE_CONTAINER/$VHD_FILE"

  ########################################################
  # this part required until we are in Azure marketplace #
  ########################################################

  log "Creating VCL OS disk..."
  $DR az disk create                  \
    --name VCL-disk                   \
    --location $LOCATION              \
    --resource-group $resource_group  \
    --source $SAS_URI 1>/dev/null     \
    || { log "Disk creation failed. Please check that you have the latest version of the install script." && exit 1; }
  log "Deploying VCL..."
  $DR az deployment group create --template-file vcl-vm.json --parameters tmp_vcl_parameters.json --resource-group $resource_group >/dev/null \
    || { log "Deployment failed. Please check that you have the latest version of the install script." && exit 1; }

  STORAGE_CONTAINER="$($DR az deployment group list --resource-group $NAME \
    | grep vclstorage \
    | grep -i resourcename \
    | head -n 1 | cut -d ":" -f 2 | tr -d "[:space:],\",\,")"
  $DR az vm boot-diagnostics enable --name vcl --resource-group $resource_group --storage "https://$STORAGE_CONTAINER.blob.core.windows.net/" >/dev/null \
    || { echo "Couldn't enable boot diagnostics on VCL."; }

  rm tmp_*

  cd ..
}

install_gcp() {
  log "Creating an Google Cloud Platform deployment for TOSIBOX(R) Virtual Central Lock"

  cd $GCP_DIR
  tag="$NAME"

  $DR gcloud services enable deploymentmanager.googleapis.com
  $DR gcloud services enable cloudresourcemanager.googleapis.com
  $DR gcloud services enable cloudbuild.googleapis.com
  #gcloud services enable computeengine.googleapis.com

  cp config.yaml tmp_config.yaml

  sed -i -e "s/placeholder/$tag/g" tmp_config.yaml

  #Create resourcres
  $DR gcloud deployment-manager deployments create $tag --config tmp_config.yaml

  rm tmp_*

  #Remove external ips if needed
  #gcloud compute instances delete-access-config instance_name --access-config-name='External NAT'

  cd ..
}

get_ip() {
  if [ "$SOURCE_IP" = "auto" ]; then
    SOURCE_IP="$(curl -s https://ipinfo.io/ip)/32"
  else
    while true; do
      log "If you want to get your IP automatically from ipinfo.io, use switch -i auto"
      read -p "Enter your source IP address (default 0.0.0.0/0 to allow all)" ip

      if [ $ip ]; then
        while true; do
          if valid_ip $ip; then
            SOURCE_IP=$ip"/32"
            break;
          else
            read -p "Enter valid IP address" ip
          fi
        done
      else
        SOURCE_IP="0.0.0.0/0"
        break;
      fi
    done
  fi

  log "Allowed IP-address is: $SOURCE_IP"
  log
}

get_domain() {
  case $VARIANT in
    aws)
      deployment=$NAME
      DOMAIN=$(aws --profile $_AWS_PROFILE --region $LOCATION cloudformation describe-stacks \
      --stack-name "$deployment" | grep -A1 "VclPublicDnsName" | tail -1 | awk -F: '{gsub(/ *"*/, "", $2); print $2}')
    ;;
    az*)
      resource_group=$NAME
      DOMAIN=$(az network public-ip show --resource-group $resource_group --name vcl-ip \
        | grep "ipAddress" \
        | awk -F: '{gsub(/ *"*\,*/, "", $2); print $2}')
    ;;
    gcp) read "Give your GCP Instance domain" DOMAIN ;;
    *) log "Variant not recognized. Try again." ;
            exit ;;
  esac
}

validate_license() {
  license="$1"
  if [ ${#license} -ne 25 ]; then
    return 0
  fi
  version=$(echo $license | cut -c1-1)
  id=$(echo $license | cut -c2-9)
  passwd=$(echo $license | cut -c10-24)
  checksum=$(echo $license | cut -c25-25)
  calculated=$(echo -n $version$id$passwd | $MD5 | cut -c1-1)

  if [ $calculated != $checksum ]; then
    return 0
  else
    return 1
  fi
}

get_activation_code() {

  if [ $ACTIVATE ] && [ $LICENSE ]; then
    log Using activation code: $LICENSE
  else
    read -p "Enter activation code (empty to cancel): " LICENSE
  fi

  validate_license $LICENSE
  LICENSE_VALID=$?
  [ "$LICENSE_VALID" -eq 0 ] && log "Invalid activation code." >&2 && exit 1
  [ -z $LICENSE ] && log "cancelled." && exit 1
  log
}

get_remote_matching_code() {
  CODE=""

  log "Please wait until the VCL system restarts (may take 5-10 minutes)."
  log-ne "Waiting for the remote matching code."

  case $VARIANT in
    aws)
      deployment=$NAME

      aws --profile $_AWS_PROFILE --region $LOCATION cloudformation describe-stacks --stack-name "$deployment" >/dev/null
      FAIL=$?
      if [ $FAIL -ne 0 ]; then
        log
        log "Failed to find the stack: $deployment"
        exit 1
      fi

      AWS_INSTANCE_ID=$(aws --profile $_AWS_PROFILE --region $LOCATION cloudformation describe-stacks \
      --stack-name "$deployment" | grep -A1 "VclInstanceID" | tail -1 | awk -F: '{gsub(/ *"*/, "", $2); print $2}')

      if [ -z $AWS_INSTANCE_ID ]; then
        log
        log "Failed to find Instance-id for the VCL."
        log "Exiting.."
        exit 1
      fi

      while [ -z "$CODE" ]; do
        CODE=$(aws --profile $_AWS_PROFILE --region $LOCATION ec2  get-console-output --instance-id $AWS_INSTANCE_ID --output text | grep "remote matching code" | tail -1)
        log-ne "."
        sleep 5 &>/dev/null
      done
    ;;
    az)
      while [ -z "$CODE" ]; do
        CODE=$(az vm boot-diagnostics get-boot-log --name vcl --resource-group $NAME 2>/dev/null | strings | grep "remote matching code" | tail -1)
        log-ne "."
        sleep 5 &>/dev/null
      done
    ;;
    gcp) echo not implemented. ;;
  esac
  log
  log
  log $CODE
}

get_variant() {
  if [ $VARIANT ]; then
    log Using $VARIANT for deployment..
  else
    log "Select the cloud service (number) to be used."
    select variant in aws az
    do
      VARIANT=$variant
      [ -z $VARIANT ] || break;
      log Please enter valid number.
    done
    #read -p "Enter cloud service name (aws/az - empty to cancel): " VARIANT
  fi

  case $VARIANT in
    aws)    [ ! -x "$(command -v aws)" ] && log "Please install 'aws' command and run again. Exiting." && exit 1 ;;
    az*)    [ ! -x "$(command -v az)" ] && log "Please install 'az' command and run again. Exiting." && exit 1 ;;
    gcp)    [ ! -x "$(command -v gcloud)" ] && log "Please install 'gcloud' command and run again. Exiting." && exit 1 ;;

    *)      log Variant not recognized. Exiting. ; exit 1;;
  esac

  [ ! -d $VARIANT ] && "Directory $VARIANT does not exists. Please run the install script with the '$VARIANT' directory with the templates."
}

get_deployment_name() {
  if [ $NAME ]; then
    log Name: $NAME given..
  else
    read -p "Enter a unique name for your deployment in lowercase (empty to cancel): " NAME
  fi

  NAME=$(echo $NAME | awk '{print tolower($0)}')

  [ -z $NAME ] && log "cancelled." && exit 1

  log Using name: $NAME for deployment..

  log
}

get_location() {

  location_term="location"

  case $VARIANT in
    aws) LOCATIONS=$(aws --profile $_AWS_PROFILE ec2 describe-regions | grep "RegionName" | awk -F: '{gsub(/ *"*,*/, "", $2); print $2}'); location_term="region" ;;
    az*) LOCATIONS=$(az account list-locations | grep "name" | awk -F: '{gsub(/ *"*,*/, "", $2); print $2}' | sort | uniq); location_term="location";;
    gcp) echo not implemented ;;
    *) log Variant not recognized. Exiting. ; exit ;;
  esac

  log "Enter the $location_term (number) for your instance:"

  select location in $LOCATIONS
  do
    LOCATION=$location
    [ -z $LOCATION ] || break;
    log Please enter valid number.
  done

  log "Using $location_term: $LOCATION"

  [ "$VARIANT" == "aws" ] && [ "$LOCATION" ] && get_availability_zone
}

get_availability_zone() {
  log "Enter the availability zone (number):"
  ZONES=$(aws --profile $_AWS_PROFILE --region $LOCATION ec2 describe-availability-zones | grep "ZoneName" | awk -F: '{gsub(/ *"*,*/, "", $2); print $2}')

  select zone in $ZONES
  do
    AWS_AVAILABILITY_ZONE=$zone
    [ -z $AWS_AVAILABILITY_ZONE ] || break;
    log Please enter valid number.
  done

  log "Using availability zone: $AWS_AVAILABILITY_ZONE"
}


activate() {
  get_deployment_name
  get_variant
  get_activation_code
  get_domain

  ACTIVATION_URL="http://"$DOMAIN"/cgi-bin/vcl-activation"
  ACTIVATION_STATUS_URL="http://"$DOMAIN"/cgi-bin/vcl-activation-status"

  log The activation url is: $ACTIVATION_URL

  #read -p "Press a key when ready to activate."

  log "Activating license.."

  $DR curl $ACTIVATION_URL"?wan_dhcp=1&wan_ip=&wan_netmask=&wan_gateway=&wan_dns=&license_key="$LICENSE"&_=1111" >/dev/null
    #$DR curl $ACTIVATION_URL"?wan_dhcp=true&license_key="$LICENSE
  #"&_=1584923450731""

  PROGRESS="progress"

  log-ne "\nWaiting until VCL installs."

  while [ $PROGRESS ]; do
    status=$(curl  $ACTIVATION_STATUS_URL 2>/dev/null | awk -F'"' '{print $4}')

    case $status in
      not_started) log-ne "\nActivation not started." ;;
      not_updated) log-ne "." ;;
      activating) log-ne "\nActivation in progress." ;;
      no_connection) log-ne "\nNo internet connection." ;;
      installing_license) log-ne "\nInstalling license." ;;
      incorrect_license) log-ne "\nLicense was incorrect, please try again."; exit 1 ;;
      license_used) log-ne "\nLicense has been already used."; exit 1 ;;
      downloading_software) log-ne "\nDownloading software." ;;
      installing_software) log-ne "\nInstalling software." ;;
      post_install) log-ne "\nInstalling software." ;;
      rebooting) log-ne "\nActivation completed, rebooting."; PROGRESS= ;;
      error) log-ne "\nSomething went wrong, please try again."; exit 1 ;;
      invalid_network_settings) log-ne "\nInvalid or missing network settings." ;;
      license_dl_failed) log-ne "\nLicense download failed."; exit 1 ;;
      license_install_failed) log-ne "\nFailed to install license."; exit 1 ;;
      package_dl_failed) log-ne "\nFailed to download software."; exit 1 ;;
      package_install_failed) log-ne "\nFailed to install software."; exit 1 ;;
      license_activation_failed) log-ne "\nLicense activation failed."; exit 1 ;;
      remove_license_failed) log-ne "\nNetwork error, cannot remove used license from license pool."; exit 1 ;;
      *) log-ne "\nWaiting until the VCL restarts.\n"; PROGRESS= ;;
    esac
    sleep 1 &>/dev/null
  done
}


install() {
  yamlfile="vcl.yaml"

  # Setup WebUI port
  log
  log "======================================================="
  log "STEP 1: TOSIBOX(R) Virtual Central Lock deployment"
  log "In order to install VCL, you need to enter the"
  log "activation code that can be ordered from:"
  log "https://www.tosibox.com/virtualcentrallocklite/"
  log
  log "Notice, that you need TOSIBOX(R) Key in order to start"
  log "using TOSIBOX(R) Ecosystem."
  log
  log "You may purchase TOSIBOX(R) Key from:"
  log " - https://www.tosibox.com/contact-us/distributors/"
  log " - https://www.verkkokauppa.com/ (search tosibox)"
  log "======================================================="
  log


  get_deployment_name
  get_ip
  get_variant

  case $VARIANT in
    aws)    install_aws ;;
    az*)  install_azure ;;
    gcp)    install_gcp ;;

    *)      log Variant not recognized. Try again. ;
            exit ;;
  esac

  log
  log "======================================================="
  log "STEP 2: Activation"
  log "Next we start activation of the Virtual Central Lock"
  log "This will phase take about 10 minutes and as it will"
  log "Download the latest software packages from TOSIBOX"
  log "software repository."
  log "======================================================="
  log


  activate

  log
  log "======================================================="
  log "STEP 3: Remote matching"
  log "In order to start using the Virtual Central Lock,"
  log "start your TOSIBOX(R) Key and select:"
  log
  log " Devices -> Remote Matching"
  log
  log "After that enter the remote maching code, and"
  log "access your TOSIBOX(R) Virtual Central Lock by"
  log "connecting your TOSIBOX(R) Key and accessing the"
  log "admin user interface."
  log "======================================================="
  log


  get_remote_matching_code

  log "Finished."
}

uninstall() {

  get_deployment_name
  get_variant

  case $VARIANT in
    aws)    uninstall_aws ;;
    az*)  uninstall_azure ;;
    gcp)    uninstall_gcp ;;

    *)      log Variant not recognized. Try again. ;
            exit ;;
  esac

  exit
}

uninstall_aws() {
  deployment=$NAME #"-stack"
  bucket=$NAME #"-bucket"

  get_aws_profile
  get_location


  $DR aws --profile $_AWS_PROFILE --region $LOCATION cloudformation delete-stack --stack-name "$deployment"

  status="DELETE_IN_PROGRESS"

  log-ne "Deleting the VCL CloudFormation deployment: $NAME"

  while [ "$status" ]
  do
      status=$(aws --profile $_AWS_PROFILE --region $LOCATION cloudformation describe-stacks --stack-name "$deployment" 2>/dev/null | grep "DELETE_IN_PROGRESS")
      log-ne "."
      sleep 1 &>/dev/null
  done

  log
  log "Deleting the deployment bucket: $bucket"
  $DR aws --profile $_AWS_PROFILE --region $LOCATION s3 rb --force "s3://$bucket"

}

uninstall_azure() {
  get_location
  log "Deleting the VCL Azure deployment: $NAME"
  $DR az group delete --name $NAME
}

uninstall_gcp() {
  get_location
  log "Deleting the VCL Google Cloud Platform deployment: $NAME"
  tag=$NAME
  $DR gcloud deployment-manager deployments delete $tag
}

if [ "$DR" = "$DRYRUN_ECHO" ]; then
  log
  log RUNNING IN DRYRUN MODE. This will not cause changes to the system.
  $DR commands run in dryrun mode are shown in red.
  log Commands that output to /dev/null may not be visible.
  # wizards check that there's only one parameter, but we want to run wizards also in dryrun mode
  PARAMCOUNT=$(($PARAMCOUNT-1));
fi

if [ $INSTALL ]; then
  install
  exit
elif [ $UNINSTALL ]; then
  uninstall
  exit
elif [ $ACTIVATE ]; then
  get_deployment_name
  get_variant

  activate
  exit
elif [ $MATCH ]; then
  get_deployment_name
  get_variant

  get_remote_matching_code
  exit
fi
